@@ -1,12 +1,11 @@
 """ Generate modern Python clients from OpenAPI """
 from __future__ import annotations
 
-import json
 import shutil
 import subprocess
-from importlib.metadata import version
+import sys
 from pathlib import Path
-from typing import Any, Dict, Optional
+from typing import Any, Dict, Optional, Sequence
 
 import httpx
 import yaml
@@ -14,14 +13,21 @@ from jinja2 import Environment, PackageLoader
 
 from openapi_python_client import utils
 
-from .openapi_parser import OpenAPI, import_string_from_reference
+from .parser import GeneratorData, import_string_from_reference
+from .parser.errors import GeneratorError
+
+if sys.version_info.minor == 7:  # version did not exist in 3.7, need to use a backport
+    from importlib_metadata import version  # type: ignore
+else:
+    from importlib.metadata import version  # type: ignore
+
 
 __version__ = version(__package__)
 
 
 def _get_project_for_url_or_path(url: Optional[str], path: Optional[Path]) -> _Project:
-    data_dict = _get_json(url=url, path=path)
-    openapi = OpenAPI.from_dict(data_dict)
+    data_dict = _get_document(url=url, path=path)
+    openapi = GeneratorData.from_dict(data_dict)
     return _Project(openapi=openapi)
 
 
@@ -30,43 +36,53 @@ def load_config(*, path: Path) -> None:
     config_data = yaml.safe_load(path.read_text())
 
     if "class_overrides" in config_data:
-        from .openapi_parser import reference
+        from .parser import reference
 
         for class_name, class_data in config_data["class_overrides"].items():
             reference.class_overrides[class_name] = reference.Reference(**class_data)
 
 
-def create_new_client(*, url: Optional[str], path: Optional[Path]) -> None:
-    """ Generate the client library """
+def create_new_client(*, url: Optional[str], path: Optional[Path]) -> Sequence[GeneratorError]:
+    """
+    Generate the client library
+
+    Returns:
+         A list containing any errors encountered when generating.
+    """
     project = _get_project_for_url_or_path(url=url, path=path)
-    project.build()
+    return project.build()
 
 
-def update_existing_client(*, url: Optional[str], path: Optional[Path]) -> None:
-    """ Update an existing client library """
+def update_existing_client(*, url: Optional[str], path: Optional[Path]) -> Sequence[GeneratorError]:
+    """
+    Update an existing client library
+
+    Returns:
+         A list containing any errors encountered when generating.
+    """
     project = _get_project_for_url_or_path(url=url, path=path)
-    project.update()
+    return project.update()
 
 
-def _get_json(*, url: Optional[str], path: Optional[Path]) -> Dict[str, Any]:
-    json_bytes: bytes
+def _get_document(*, url: Optional[str], path: Optional[Path]) -> Dict[str, Any]:
+    yaml_bytes: bytes
     if url is not None and path is not None:
         raise ValueError("Provide URL or Path, not both.")
-    elif url is not None:
+    if url is not None:
         response = httpx.get(url)
-        json_bytes = response.content
+        yaml_bytes = response.content
     elif path is not None:
-        json_bytes = path.read_bytes()
+        yaml_bytes = path.read_bytes()
     else:
         raise ValueError("No URL or Path provided")
-    return json.loads(json_bytes)
+    return yaml.safe_load(yaml_bytes)
 
 
 class _Project:
     TEMPLATE_FILTERS = {"snakecase": utils.snake_case}
 
-    def __init__(self, *, openapi: OpenAPI) -> None:
-        self.openapi: OpenAPI = openapi
+    def __init__(self, *, openapi: GeneratorData) -> None:
+        self.openapi: GeneratorData = openapi
         self.env: Environment = Environment(loader=PackageLoader(__package__), trim_blocks=True, lstrip_blocks=True)
 
         self.project_name: str = f"{openapi.title.replace(' ', '-').lower()}-client"
@@ -78,18 +94,22 @@ class _Project:
 
         self.env.filters.update(self.TEMPLATE_FILTERS)
 
-    def build(self) -> None:
+    def build(self) -> Sequence[GeneratorError]:
         """ Create the project from templates """
 
         print(f"Generating {self.project_name}")
-        self.project_dir.mkdir()
+        try:
+            self.project_dir.mkdir()
+        except FileExistsError:
+            return [GeneratorError(detail="Directory already exists. Delete it or use the update command.")]
         self._create_package()
         self._build_metadata()
         self._build_models()
         self._build_api()
         self._reformat()
+        return self._get_errors()
 
-    def update(self) -> None:
+    def update(self) -> Sequence[GeneratorError]:
         """ Update an existing project """
 
         if not self.package_dir.is_dir():
@@ -100,10 +120,20 @@ class _Project:
         self._build_models()
         self._build_api()
         self._reformat()
+        return self._get_errors()
 
     def _reformat(self) -> None:
-        subprocess.run("isort --recursive --apply", cwd=self.project_dir, shell=True)
-        subprocess.run("black .", cwd=self.project_dir, shell=True)
+        subprocess.run(
+            "isort .", cwd=self.project_dir, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+        )
+        subprocess.run("black .", cwd=self.project_dir, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+
+    def _get_errors(self) -> Sequence[GeneratorError]:
+        errors = []
+        for collection in self.openapi.endpoint_collections_by_tag.values():
+            errors.extend(collection.parse_errors)
+        errors.extend(self.openapi.schemas.errors)
+        return errors
 
     def _create_package(self) -> None:
         self.package_dir.mkdir()
@@ -147,11 +177,15 @@ class _Project:
         models_init = models_dir / "__init__.py"
         imports = []
 
+        types_template = self.env.get_template("types.py")
+        types_path = models_dir / "types.py"
+        types_path.write_text(types_template.render())
+
         model_template = self.env.get_template("model.pyi")
-        for schema in self.openapi.schemas.values():
-            module_path = models_dir / f"{schema.reference.module_name}.py"
-            module_path.write_text(model_template.render(schema=schema))
-            imports.append(import_string_from_reference(schema.reference))
+        for model in self.openapi.schemas.models.values():
+            module_path = models_dir / f"{model.reference.module_name}.py"
+            module_path.write_text(model_template.render(model=model))
+            imports.append(import_string_from_reference(model.reference))
 
         # Generate enums
         enum_template = self.env.get_template("enum.pyi")
