@@ -1,11 +1,10 @@
 import pathlib
-from contextlib import contextmanager
 from pprint import pformat
-from typing import Generator, Optional
+from typing import Optional, Sequence
 
 import typer
 
-from openapi_python_client.openapi_parser.errors import MultipleParseError, ParseError
+from openapi_python_client.parser.errors import ErrorLevel, GeneratorError, ParseError
 
 app = typer.Typer()
 
@@ -26,7 +25,7 @@ def _process_config(path: Optional[pathlib.Path]) -> None:
 
     try:
         load_config(path=path)
-    except:
+    except:  # noqa
         raise typer.BadParameter("Unable to parse config")
 
 
@@ -42,35 +41,56 @@ def cli(
     pass
 
 
-def _print_parser_error(e: ParseError) -> None:
-    formatted_data = pformat(e.data)
-    typer.secho(e.header, bold=True, fg=typer.colors.BRIGHT_RED, err=True)
-    typer.secho(formatted_data, fg=typer.colors.RED, err=True)
-    if e.message:
-        typer.secho(e.message, fg=typer.colors.BRIGHT_RED, err=True)
-    gh_link = typer.style(
-        "https://github.com/triaxtec/openapi-python-client/issues/new/choose", fg=typer.colors.BRIGHT_BLUE
-    )
-    typer.secho(f"Please open an issue at {gh_link}", fg=typer.colors.RED, err=True)
+def _print_parser_error(e: GeneratorError, color: str) -> None:
+    typer.secho(e.header, bold=True, fg=color, err=True)
+    if e.detail:
+        typer.secho(e.detail, fg=color, err=True)
+
+    if isinstance(e, ParseError) and e.data is not None:
+        formatted_data = pformat(e.data)
+        typer.secho(formatted_data, fg=color, err=True)
+
     typer.secho()
 
 
-@contextmanager
-def handle_errors() -> Generator[None, None, None]:
+def handle_errors(errors: Sequence[GeneratorError]) -> None:
     """ Turn custom errors into formatted error messages """
-    try:
-        yield
-    except ParseError as e:
-        _print_parser_error(e)
-        raise typer.Exit(code=1)
-    except MultipleParseError as e:
-        typer.secho("MULTIPLE ERRORS WHILE PARSING:", underline=True, bold=True, fg=typer.colors.BRIGHT_RED, err=True)
-        for err in e.parse_errors:
-            _print_parser_error(err)
-        raise typer.Exit(code=1)
-    except FileExistsError:
-        typer.secho("Directory already exists. Delete it or use the update command.", fg=typer.colors.RED, err=True)
-        raise typer.Exit(code=1)
+    if len(errors) == 0:
+        return
+    color = typer.colors.YELLOW
+    for error in errors:
+        if error.level == ErrorLevel.ERROR:
+            typer.secho(
+                "Error(s) encountered while generating, client was not created",
+                underline=True,
+                bold=True,
+                fg=typer.colors.BRIGHT_RED,
+                err=True,
+            )
+            color = typer.colors.RED
+            break
+    else:
+        typer.secho(
+            "Warning(s) encountered while generating. Client was generated, but some pieces may be missing",
+            underline=True,
+            bold=True,
+            fg=typer.colors.BRIGHT_YELLOW,
+            err=True,
+        )
+
+    for err in errors:
+        _print_parser_error(err, color)
+
+    gh_link = typer.style(
+        "https://github.com/triaxtec/openapi-python-client/issues/new/choose", fg=typer.colors.BRIGHT_BLUE
+    )
+    typer.secho(
+        f"If you believe this was a mistake or this tool is missing a feature you need, "
+        f"please open an issue at {gh_link}",
+        fg=typer.colors.BLUE,
+        err=True,
+    )
+    raise typer.Exit(code=1)
 
 
 @app.command()
@@ -84,11 +104,11 @@ def generate(
     if not url and not path:
         typer.secho("You must either provide --url or --path", fg=typer.colors.RED)
         raise typer.Exit(code=1)
-    elif url and path:
+    if url and path:
         typer.secho("Provide either --url or --path, not both", fg=typer.colors.RED)
         raise typer.Exit(code=1)
-    with handle_errors():
-        create_new_client(url=url, path=path)
+    errors = create_new_client(url=url, path=path)
+    handle_errors(errors)
 
 
 @app.command()
@@ -102,8 +122,9 @@ def update(
     if not url and not path:
         typer.secho("You must either provide --url or --path", fg=typer.colors.RED)
         raise typer.Exit(code=1)
-    elif url and path:
+    if url and path:
         typer.secho("Provide either --url or --path, not both", fg=typer.colors.RED)
         raise typer.Exit(code=1)
-    with handle_errors():
-        update_existing_client(url=url, path=path)
+
+    errors = update_existing_client(url=url, path=path)
+    handle_errors(errors)
